"""Ring 3: Intelligence — Discovery, Consensus, Intervention, Counterfactual, Validation.

Multi-algorithm causal discovery engine with PC (constraint-based),
GES (score-based), consensus voting, d-separation falsification,
formal do-calculus intervention analysis, and counterfactual reasoning.
"""

from orxaq.intelligence.consensus import ConsensusEdge, ConsensusEngine, ConsensusResult
from orxaq.intelligence.counterfactual import (
    CounterfactualEngine,
    CounterfactualQuery,
    CounterfactualResult,
    NecessaryCausation,
    SufficientCausation,
)
from orxaq.intelligence.ges_algorithm import GESAlgorithm, GESResult
from orxaq.intelligence.independence import CIResult, PartialCorrelationTest, pearson_correlation
from orxaq.intelligence.intervention import (
    AdjustmentSet,
    AdjustmentSetFinder,
    IdentifiabilityChecker,
    IdentificationMethod,
    IdentificationResult,
    InterventionEstimator,
    InterventionResult,
    MediationAnalyzer,
    MediationResult,
    SensitivityAnalyzer,
    SensitivityResult,
)
from orxaq.intelligence.pc_algorithm import PCAlgorithm, PCResult
from orxaq.intelligence.validation import (
    DSeparationTester,
    DSepImplication,
    DSepReport,
    RefutationSuite,
)

__all__ = [
    "AdjustmentSet",
    "AdjustmentSetFinder",
    "CIResult",
    "ConsensusEdge",
    "ConsensusEngine",
    "ConsensusResult",
    "CounterfactualEngine",
    "CounterfactualQuery",
    "CounterfactualResult",
    "DSepImplication",
    "DSepReport",
    "DSeparationTester",
    "GESAlgorithm",
    "GESResult",
    "IdentifiabilityChecker",
    "IdentificationMethod",
    "IdentificationResult",
    "InterventionEstimator",
    "InterventionResult",
    "MediationAnalyzer",
    "MediationResult",
    "NecessaryCausation",
    "PCAlgorithm",
    "PCResult",
    "PartialCorrelationTest",
    "RefutationSuite",
    "SensitivityAnalyzer",
    "SensitivityResult",
    "SufficientCausation",
    "pearson_correlation",
]
