"""Skip starlette-dependent tests when starlette is not installed."""

from __future__ import annotations

import importlib.util

collect_ignore: list[str] = []

if importlib.util.find_spec("starlette") is None:
    collect_ignore.append("test_rest_api.py")
