"""Beta skin — Command Engine.

Monospace CLI-inspired terminal aesthetic.
Green-on-black, fixed-width, information-dense.
No border-radius, no shadows, no gradients.
"""

from __future__ import annotations

from orxaq.experiences.skins.base import Skin

BETA_SKIN = Skin(
    name="beta",
    display_name="Beta Command Engine",
    description="Monospace terminal aesthetic, green-on-black, keyboard-navigable",
    template_file="beta.html",
    css_variables={
        "--bg-primary": "#0a0a0a",
        "--bg-secondary": "#111111",
        "--bg-card": "#1a1a1a",
        "--text-primary": "#00ff41",
        "--text-secondary": "#00aa2a",
        "--accent-primary": "#00ff41",
        "--accent-secondary": "#ffcc00",
        "--border-color": "#333333",
        "--font-family": "'JetBrains Mono', 'Fira Code', monospace",
        "--border-radius": "0px",
        "--card-shadow": "none",
    },
)
